/* @(#)51       1.10  src/avs/fs/mmfs/ts/kernext/gpfs_fcntl.h, mmfs, avs_rcs3b, rcs3bs004a 8/12/03 21:53:53 */

/*
 * GPFS interface definitions for supporting I/O hints and directives.
 *
 * Usage: The argument to gpfs_fcntl is composed of the concatenation of
 * structures defined in this file.  The first structure must be of type
 * gpfsFcntlHeader_t.  This is immediately followed by additional
 * structures, one for each hint or directive supplied.  The totalLength
 * field of the header contains the length of all of the structures,
 * including the header itself.  Each structure is defined to be a multiple
 * of 8 bytes in length, and the highest alignment requirement of any of the
 * data types is also 8 bytes, so the compiler will not insert padding when
 * several structures are declared within an outer structure.  This makes
 * it easier to build up the necessary area before calling gpfs_fcntl.
 *
 * For example, the following code fragment first releases all cached data
 * held on behalf of a file, then tells GPFS that this node will write
 * the portion of the file with file offsets between 2G and 3G-1:
 *   struct
 *   {
 *     gpfsFcntlHeader_t hdr;
 *     gpfsClearFileCache_t rel;
 *     gpfsAccessRange_t acc;
 *   } arg;
 *
 *   arg.hdr.totalLength = sizeof(arg);
 *   arg.hdr.fcntlVersion = GPFS_FCNTL_CURRENT_VERSION;
 *   arg.hdr.fcntlReserved = 0;
 *   arg.rel.structLen = sizeof(arg.rel);
 *   arg.rel.structType = GPFS_CLEAR_FILE_CACHE;
 *   arg.acc.structLen = sizeof(arg.acc);
 *   arg.acc.structType = GPFS_ACCESS_RANGE;
 *   arg.acc.start = 2LL * 1024LL * 1024LL * 1024LL;
 *   arg.acc.length = 1024 * 1024 * 1024;
 *   arg.acc.isWrite = 1;
 *   rc = gpfs_fcntl(handle, &arg);
 *
 * If gpfs_fcntl returns an error (rc -1), errno will contain the error
 * reason, and the errorOffset field of the header will contain the offset
 * of the offending structure within the argument area.
 *
 * In general, the structures within the argument are processed in order,
 * except that data shipping directives are performed after all other hints
 * and directives.
 */

#ifndef _h_gpfs_fcntl
#define _h_gpfs_fcntl

/* Header of the parameter area passed to gpfs_fcntl */
typedef struct
{
  int           totalLength;    /* length of this structure plus the sum of
                                   the lengths of all structures in this
                                   gpfs_fcntl argument */
  int           fcntlVersion;   /* version number: GPFS_FCNTL_CURRENT_VERSION */
  int           errorOffset;    /* Returned value giving offset into parameter
                                   area of the structure to which errno
                                   pertains.  Only set if errno is set. */
  int           fcntlReserved;  /* not used, should be set to 0 */
} gpfsFcntlHeader_t;


/* Interface version number (fcntlVersion field of gpfsFcntlHeader_t) */
#define GPFS_FCNTL_CURRENT_VERSION      1

/* Maximum length of argument to gpfs_fcntl */
#define GPFS_MAX_FCNTL_LENGTH       65536


/* Definitions of structType fields for GPFS hints.  Hints can be ignored
   by GPFS without affecting correct operation, although performance might
   suffer. */
#define GPFS_ACCESS_RANGE            1001
#define GPFS_FREE_RANGE              1002
#define GPFS_MULTIPLE_ACCESS_RANGE   1003
#define GPFS_CLEAR_FILE_CACHE        1004

/* Definitions of structType fields for GPFS directives.  GPFS must honor
   directives, or return an error saying why a directive could not be
   honored. */
#define GPFS_CANCEL_HINTS            2001
#define GPFS_DATA_SHIP_START         2002
#define GPFS_DATA_SHIP_MAP           2003
#define GPFS_DATA_SHIP_STOP          2004


/* Structures for specifying the various gpfs_fnctl hints */

/* Access range hint:  The application will soon access file offsets within
   the given range, and will not access offsets outside the range.  Violating
   this hint may produce worse performance than if no hint was specified. */
typedef struct
{
  int           structLen;      /* length of this structure */
  int           structType;     /* hint identifier: GPFS_ACCESS_RANGE */
  long long     start;          /* start offset in bytes from beginning of file */
  long long     length;         /* length of range; 0 indicates to end of file */
  int           isWrite;        /* 0 - read access, 1 - write access */
  char          padding[4];
} gpfsAccessRange_t;


/* Free range hint: the application will no longer access file offsets
   within the given range, so GPFS is free to flush those file offsets from
   its cache. */
typedef struct
{
  int           structLen;      /* length of this structure */
  int           structType;     /* hint identifier: GPFS_FREE_RANGE */
  long long     start;          /* start offset in bytes from beginning of file */
  long long     length;         /* length of range; 0 indicates to end of file */
} gpfsFreeRange_t;


/* Format of accRangeArray and relRangeArray entries used by
   GPFS_MULTIPLE_ACCESS_RANGE hint */
typedef struct
{
  long long     blockNumber;    /* data block number to access */
  int           start;          /* start of range (from beginning of block) */
  int           length;         /* number of bytes in the range  */
  int           isWrite;        /* 0 - read access, 1 - write access */
  char          padding[4];
} gpfsRangeArray_t;

/* Multiple access range hint: This hint is used to drive application-defined
   prefetching and writebehind.  The application will soon access the
   portions of the blocks specified in accRangeArray, and has finished
   accessing the ranges listed in relRangeArray.  The size of a block is
   returned by stat in the st_blksize field, so offset OFF of a file is in
   block OFF/st_blksize.  Up to GPFS_MAX_RANGE_COUNT blocks may be given in
   one multiple access range hint.  Depending on the current load, GPFS may
   initiate prefetching of some or all of these.  Each range named in
   accRangeArray that is accepted for prefetching should eventually be
   released via relRangeArray, or else GPFS will stop prefetching blocks
   for this file. */
#define GPFS_MAX_RANGE_COUNT 8
typedef struct
{
  int           structLen;      /* length of this structure */
  int           structType;     /* hint identifier: GPFS_MULTIPLE_ACCESS_RANGE */
  int           accRangeCnt;    /* on input, number of ranges in accRangeArray
                                   on output, number of processed ranges (the
                                   first n of the given ranges) */
  int           relRangeCnt;    /* number of ranges in relRangeArray */
  gpfsRangeArray_t accRangeArray[GPFS_MAX_RANGE_COUNT]; /* requested ranges */
  gpfsRangeArray_t relRangeArray[GPFS_MAX_RANGE_COUNT]; /* ranges to release */
} gpfsMultipleAccessRange_t;


/* Clear file cache hint: the application expects to no longer access any
   portion of the file, so GPFS should flush and invalidate any cached
   data belonging to this file.  This may avoid synchronous cache invalidations
   on later uses of the file by other nodes. */
typedef struct
{
  int           structLen;      /* length of this structure */
  int           structType;     /* hint identifier: GPFS_CLEAR_FILE_CACHE */
} gpfsClearFileCache_t;


/* Structures for specifying the various gpfs_fnctl directives */

/* Cancel all hints: GPFS removes any hints that may have been issued
   against this file.  Does not affect the contents of the GPFS file cache.
   Note that this directive does not cancel the effect of other directives,
   such as GPFS_DATA_SHIP_START. */
typedef struct  /* cancelAccessHints hint */
{
  int           structLen;      /* length of this structure */
  int           structType;     /* hint identifier: GPFS_CANCEL_HINTS */
} gpfsCancelHints_t;


/* Initiate data shipping mode: once all participating threads have issued
   this hint for a file, GPFS enters a mode where it partitions the blocks
   of the file among a group of agent nodes.  In data shipping mode, all
   file accesses result in GPFS messages to the appropriate agent(s) to read
   or write the requested data.  Applications that perform fine-grained
   write sharing should benefit most from data shipping, since GPFS cache
   invalidations will be avoided.  Because an application level read or
   write may be split across several agents, Posix read/write file atomicity
   is not enforced while in data shipping mode. */
typedef struct
{
  int           structLen;      /* length of this structure */
  int           structType;     /* directive identifier: GPFS_DATA_SHIP_START */
  int           numInstances;   /* number of open file instances collaborating
                                   to operate on the file.  These may be on
                                   any number of nodes. */
  int           reserved;       /* not used, should be set to 0 */
} gpfsDataShipStart_t;


/* Specify the agent mapping for data shipping: tells GPFS which agent
   nodes to use for data shipping.  This directive can only appear in a
   gpfs_fcntl call that also gives the GPFS_DATA_SHIP_START directive.  If
   this directive is not used, the agents will be exactly the nodes on which
   the GPFS_DATA_SHIP_START directive was given, and the partition will be a
   round-robin distribution of the blocks of the file to the agent nodes.
   There are two forms of this directive.  The gpfsDataShipMap_t structure
   contains enough room for up to GPFS_MAX_DS_AGENT_NODES agent nodes.  If
   this is insufficient, the variable-size structure gpfsDataShipMapVariable_t
   may be used instead.  GPFS will infer the actual size of the agentNodeNumber
   array in the gpfsDataShipMapVariable_t structure from its structLen
   field.  To build the argument to gpfs_fcntl when gpfsDataShipMapVariable_t
   is used, the application needs to dynamically allocate space for all of
   the contiguous structures to be passed to gpfs_fcntl.  The following code
   fragment illustrates how this might be done:

     char * p;
     gpfsFcntlHeader_t * hdrP;
     gpfsDataShipStart_t * startP;
     gpfsDataShipMapVariable_t * mapP;
     int mapSize = GPFS_DATA_SHIP_MAP_VARIABLE_SIZE(nAgents);

     p = malloc(sizeof(gpfsFcntlHeader_t) + sizeof(gpfsDataShipStart_t) + mapSize);
     hdrP = (gpfsFcntlHeader_t *) p;
     startP = (gpfsDataShipStart_t *) (p + sizeof(gpfsFcntlHeader_t));
     mapP = (gpfsDataShipMapVariable_t *) (p + sizeof(gpfsFcntlHeader_t) +
                                           sizeof(gpfsDataShipStart_t));
     hdrP->totalLength = sizeof(gpfsFcntlHeader_t) +
                         sizeof(gpfsDataShipStart_t) + mapSize;
     ... fill in other fields in *hdrP
     startP->structLen = sizeof(gpfsDataShipStart_t);
     ... fill in other fields in *startP
     mapP->structLen = mapSize;
     ... fill in other fields in mapP, including nAgents entries in the
       mapP->agentNodeNumber array
     rc = gpfs_fcntl(handle, hdrP);
     free(p);

  As the example code shows, using gpfsDataShipMapVariable_t is more awkward
  than using gpfsDataShipMap_t, since the latter can simply be embedded
  within a larger structure that contains the necessary gpfsFcntlHeader_t
  and gpfsDataShipStart_t structures. */
#define GPFS_MAX_DS_AGENT_NODES 2048
typedef struct
{
  int           structLen;      /* length of this structure */
  int           structType;     /* directive identifier: GPFS_DATA_SHIP_MAP */
  int           partitionSize;  /* number of contiguous bytes per server */
  int           agentCount;     /* number of entries used in the
                                   agentNodeNumber array */
  int           agentNodeNumber[GPFS_MAX_DS_AGENT_NODES]; 
                                /* data ship agent node numbers, using 
                                   GPFS configuration data repository
                                   node numbers */
} gpfsDataShipMap_t;

typedef struct
{
  int           structLen;      /* length of this structure */
  int           structType;     /* directive identifier: GPFS_DATA_SHIP_MAP */
  int           partitionSize;  /* number of contiguous bytes per server */
  int           agentCount;     /* number of entries used in the
                                   agentNodeNumber array */
  int           agentNodeNumber[2]; 
                                /* data ship agent node numbers, using 
                                   GPFS configuration data repository
                                   node numbers.  The actual size of this
                                   array will be inferred from structLen.
                                   The number of elements in the array must
                                   be even, so that the total structure size
                                   is a multiple of 8.  The number of
                                   entries used in the array is given by
                                   agentCount, and may be any positive value
                                   less than or equal to the size of the
                                   array. */
} gpfsDataShipMapVariable_t;

/* Compute the size in bytes of a gpfsDataShipMapVariable_t structure large
   enough to hold _nAgents data shipping agent node numbers */
#define GPFS_DATA_SHIP_MAP_VARIABLE_SIZE(_nAgents) \
  ( sizeof(gpfsDataShipMapVariable_t) - 2*sizeof(int) + \
    ((((_nAgents)+1)/2)*2)*sizeof(int) )


/* Terminate data shipping: waits for all threads that issued the
   GPFS_DATA_SHIP_START directive to issue this directive, then leaves data
   shipping mode. */
typedef struct
{
  int           structLen;      /* length of this structure */
  int           structType;     /* directive identifier: GPFS_DATA_SHIP_STOP */
} gpfsDataShipStop_t;


/* NAME:        gpfs_fcntl()
 *
 * FUNCTION:    Pass hints and directives to GPFS on behalf of an open file
 *
 * Returns:      0      Successful
 *              -1      Failure
 *
 * Errno:       ENOSYS  Function not available
 *              EBADF   Bad file handle
 *              EINVAL  Not a GPFS file
 *              EINVAL  Not a regular file
 *              EINVAL  Ill-formed hint or directive
 *              E2BIG   Argument longer than GPFS_MAX_FCNTL_LENGTH
 */
#ifdef __cplusplus
extern "C"
{
#endif
int gpfs_fcntl(int fileDesc,            /* Open file descriptor */
                void* fcntlArgP);       /* argument list */
#ifdef __cplusplus
}
#endif

#endif /* _h_gpfs_fcntl */
